FileModuleResolver

Struct FileModuleResolver 

pub struct FileModuleResolver { /* private fields */ }
Available on crate feature rhai only.
A module resolution service that loads module script files from the file system.

Caching

Resolved Modules are cached internally so script files are not reloaded and recompiled for subsequent requests.

Use clear_cache or clear_cache_for_path to clear the internal cache.

Namespace

When a function within a script file module is called, all functions defined within the same script are available, evan private ones. In other words, functions defined in a module script can always cross-call each other.

Example

use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;

// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.x'.
let resolver = FileModuleResolver::new_with_path_and_extension("./scripts", "x");

let mut engine = Engine::new();

engine.set_module_resolver(resolver);

Implementations

Source

impl FileModuleResolver

pub fn new() -> FileModuleResolver

Create a new FileModuleResolver with the current directory as base path.

The default extension is .rhai.

Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;

// Create a new 'FileModuleResolver' loading scripts from the current directory
// with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new();

let mut engine = Engine::new();
engine.set_module_resolver(resolver);
pub fn new_with_path(path: impl Into<PathBuf>) -> FileModuleResolver

Create a new FileModuleResolver with a specific base path.

The default extension is .rhai.

Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;

// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new_with_path("./scripts");

let mut engine = Engine::new();
engine.set_module_resolver(resolver);
pub fn new_with_extension( extension: impl Into<SmartString<LazyCompact>>, ) -> FileModuleResolver

Create a new FileModuleResolver with a file extension.

Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;

// Create a new 'FileModuleResolver' loading scripts with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new_with_extension("rhai");

let mut engine = Engine::new();
engine.set_module_resolver(resolver);
pub fn new_with_path_and_extension( path: impl Into<PathBuf>, extension: impl Into<SmartString<LazyCompact>>, ) -> FileModuleResolver

Create a new FileModuleResolver with a specific base path and file extension.

Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;

// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.x'.
let resolver = FileModuleResolver::new_with_path_and_extension("./scripts", "x");

let mut engine = Engine::new();
engine.set_module_resolver(resolver);
pub fn base_path(&self) -> Option<&Path>

Get the base path for script files.

pub fn set_base_path( &mut self, path: impl Into<PathBuf>, ) -> &mut FileModuleResolver

Set the base path for script files.

pub fn extension(&self) -> &str

Get the script file extension.

pub fn set_extension( &mut self, extension: impl Into<SmartString<LazyCompact>>, ) -> &mut FileModuleResolver

Set the script file extension.

pub const fn scope(&self) -> &Scope<'_>

Get a reference to the file module resolver’s scope.

The scope is used for compiling module scripts.

pub fn set_scope(&mut self, scope: Scope<'static>)

Set the file module resolver’s scope.

The scope is used for compiling module scripts.

pub fn scope_mut(&mut self) -> &mut Scope<'static>

Get a mutable reference to the file module resolver’s scope.

The scope is used for compiling module scripts.

pub fn enable_cache(&mut self, enable: bool) -> &mut FileModuleResolver

Enable/disable the cache.

pub const fn is_cache_enabled(&self) -> bool

Is the cache enabled?

pub fn is_cached(&self, path: impl AsRef<Path>) -> bool

Is a particular path cached?

pub fn clear_cache(&mut self) -> &mut FileModuleResolver

Empty the internal cache.

pub fn clear_cache_for_path( &mut self, path: impl AsRef<Path>, ) -> Option<Arc<Module>>

Remove the specified path from internal cache.

The next time this path is resolved, the script file will be loaded once again.

pub fn get_file_path(&self, path: &str, source_path: Option<&Path>) -> PathBuf

Construct a full file path.