ModuleResolversCollection

Struct ModuleResolversCollection 

pub struct ModuleResolversCollection(/* private fields */);
Available on crate feature rhai only.
Module resolution service that holds a collection of module resolvers, to be searched in sequential order.

Example

use rhai::{Engine, Module};
use rhai::module_resolvers::{StaticModuleResolver, ModuleResolversCollection};

let mut collection = ModuleResolversCollection::new();

let resolver = StaticModuleResolver::new();
collection.push(resolver);

let mut engine = Engine::new();
engine.set_module_resolver(collection);

Implementations

Source

impl ModuleResolversCollection

pub const fn new() -> ModuleResolversCollection

Create a new ModuleResolversCollection.

Example
use rhai::{Engine, Module};
use rhai::module_resolvers::{StaticModuleResolver, ModuleResolversCollection};

let mut collection = ModuleResolversCollection::new();

let resolver = StaticModuleResolver::new();
collection.push(resolver);

let mut engine = Engine::new();
engine.set_module_resolver(collection);
pub fn push( &mut self, resolver: impl ModuleResolver + 'static, ) -> &mut ModuleResolversCollection

Append a module resolver to the end.

pub fn insert( &mut self, index: usize, resolver: impl ModuleResolver + 'static, ) -> &mut ModuleResolversCollection

Insert a module resolver to an offset index.

Panics

Panics if the index is out of bounds.

pub fn pop(&mut self) -> Option<Box<dyn ModuleResolver>>

Remove the last module resolver from the end, if any.

pub fn remove(&mut self, index: usize) -> Box<dyn ModuleResolver>

Remove a module resolver at an offset index.

Panics

Panics if the index is out of bounds.

pub fn iter(&self) -> impl Iterator<Item = &dyn ModuleResolver>

Get an iterator of all the module resolvers.

pub fn clear(&mut self) -> &mut ModuleResolversCollection

Remove all module resolvers.

pub fn is_empty(&self) -> bool

Returns true if this ModuleResolversCollection contains no module resolvers.

pub fn len(&self) -> usize

Get the number of module resolvers in this ModuleResolversCollection.

pub fn append( &mut self, other: ModuleResolversCollection, ) -> &mut ModuleResolversCollection

Add another ModuleResolversCollection to the end of this collection. The other ModuleResolversCollection is consumed.