Skip to main contentModule engine
Source - AST
- Compiled AST (abstract syntax tree) of a Rhai script.
- Dynamic
- Dynamic type containing any value.
- Engine
- Rhai main scripting engine.
- FileModuleResolver
- A module resolution service that loads module script files from the file system.
- ImmutableString
- The system immutable string type.
- Module
- A module which may contain variables, sub-modules, external Rust functions,
and/or script-defined functions.
- ModuleResolversCollection
- Module resolution service that holds a collection of module resolvers,
to be searched in sequential order.
- ParseError
- Error when parsing a script.
- Scope
- Type containing information about the current scope. Useful for keeping state between
Engine evaluation runs. - Script
- ASTNode
- (internals) An
AST node, consisting of either an Expr or a Stmt.
Exported under the internals feature only. - Expr
- (internals) An expression sub-tree.
Exported under the
internals feature only. - Stmt
- (internals) A statement.
Exported under the
internals feature only.
- base64_decode
- to_dynamic
- Serialize a Rust type that implements
serde::Serialize into a Dynamic. - url_encode
- Array
- Variable-sized array of
Dynamic values. - Map
- A dictionary of
Dynamic values with string keys.