Stmt

Enum Stmt 

#[non_exhaustive]
pub enum Stmt {
Show 17 variants Noop(Position), If(Box<FlowControl>, Position), Switch(Box<(Expr, SwitchCasesCollection)>, Position), While(Box<FlowControl>, Position), Do(Box<FlowControl>, ASTFlags, Position), For(Box<(Ident, Option<Ident>, FlowControl)>, Position), Var(Box<(Ident, Expr, Option<NonZero<usize>>)>, ASTFlags, Position), Assignment(Box<(OpAssignment, BinaryExpr)>), FnCall(Box<FnCallExpr>, Position), Block(Box<StmtBlock>), TryCatch(Box<FlowControl>, Position), Expr(Box<Expr>), BreakLoop(Option<Box<Expr>>, ASTFlags, Position), Return(Option<Box<Expr>>, ASTFlags, Position), Import(Box<(Expr, Ident)>, Position), Export(Box<(Ident, Ident)>, Position), Share(Box<SmallVec<[(Ident, Option<NonZero<usize>>); 5]>>),
}
Available on crate feature rhai only.
(internals) A statement. Exported under the internals feature only.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Noop(Position)

No-op.



If(Box<FlowControl>, Position)

if expr { stmt } else { stmt }



Switch(Box<(Expr, SwitchCasesCollection)>, Position)

switch expr { literal or range or _ if condition => stmt ,}

Data Structure
  1. Hash table for (condition, block)
  2. Default block
  3. List of ranges: (start, end, inclusive, condition, statement)


While(Box<FlowControl>, Position)

while expr { stmt } | loop { stmt }

If the guard expression is UNIT, then it is a loop statement.



Do(Box<FlowControl>, ASTFlags, Position)

do { stmt } while|until expr

Flags


For(Box<(Ident, Option<Ident>, FlowControl)>, Position)

for ( id , counter ) in expr { stmt }



Var(Box<(Ident, Expr, Option<NonZero<usize>>)>, ASTFlags, Position)

[export] let|const id = expr

Flags


Assignment(Box<(OpAssignment, BinaryExpr)>)

expr op= expr



FnCall(Box<FnCallExpr>, Position)

func ( expr ,)

This is a duplicate of Expr::FnCall to cover the very common pattern of a single function call forming one statement.



Block(Box<StmtBlock>)

{ stmt;}



TryCatch(Box<FlowControl>, Position)

try { stmt; … } catch ( var ) { stmt; … }



Expr(Box<Expr>)



BreakLoop(Option<Box<Expr>>, ASTFlags, Position)

continue/break expr

Flags


Return(Option<Box<Expr>>, ASTFlags, Position)

return/throw expr

Flags


Import(Box<(Expr, Ident)>, Position)

Available on non-crate feature no_module only.

import expr as alias

Not available under no_module.



Export(Box<(Ident, Ident)>, Position)

Available on non-crate feature no_module only.

export var as alias

Not available under no_module.



Share(Box<SmallVec<[(Ident, Option<NonZero<usize>>); 5]>>)

Available on non-crate feature no_closure only.

Convert a list of variables to shared.

Not available under no_closure.

Notes

This variant does not map to any language structure. It is currently only used only to convert normal variables into shared variables when they are captured by a closure.

Implementations

Source

impl Stmt

pub const fn is_noop(&self) -> bool

Is this statement Noop?

pub const fn options(&self) -> ASTFlags

Get the options of this statement.

pub fn position(&self) -> Position

Get the position of this statement.

pub fn set_position(&mut self, new_pos: Position) -> &mut Stmt

Override the position of this statement.

pub const fn returns_value(&self) -> bool

Does this statement return a value?

pub const fn is_self_terminated(&self) -> bool

Is this statement self-terminated (i.e. no need for a semicolon terminator)?

pub fn is_pure(&self) -> bool

Is this statement pure?

A pure statement has no side effects.

pub fn is_block_dependent(&self) -> bool

Does this statement’s behavior depend on its containing block?

A statement that depends on its containing block behaves differently when promoted to an upper block.

Currently only variable definitions (i.e. let and const), import/export statements, and eval calls (which may in turn define variables) fall under this category.

pub fn is_internally_pure(&self) -> bool

Is this statement pure within the containing block?

An internally pure statement only has side effects that disappear outside the block.

Currently only variable definitions (i.e. let and const) and import/export statements are internally pure, other than pure expressions.

pub const fn is_control_flow_break(&self) -> bool

Does this statement break the current control flow through the containing block?

Currently this is only true for return, throw, break and continue.

All statements following this statement will essentially be dead code.

pub fn take(&mut self) -> Stmt

Return this Stmt, replacing it with Stmt::Noop.

pub fn walk<'a>( &'a self, path: &mut Vec<ASTNode<'a>>, on_node: &mut (impl FnMut(&[ASTNode<'_>]) -> bool + ?Sized), ) -> bool

Recursively walk this statement. Return false from the callback to terminate the walk.