pub struct FileModuleResolver { /* private fields */ }rhai only.Expand description
A module resolution service that loads module script files from the file system.
Caching
Resolved Modules are cached internally so script files are not reloaded and recompiled for subsequent requests.
Use clear_cache or
clear_cache_for_path to clear the internal cache.
Namespace
When a function within a script file module is called, all functions defined within the same
script are available, evan private ones. In other words, functions defined in a module script
can always cross-call each other.
Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;
// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.x'.
let resolver = FileModuleResolver::new_with_path_and_extension("./scripts", "x");
let mut engine = Engine::new();
engine.set_module_resolver(resolver);Implementations
Sourceimpl FileModuleResolver
impl FileModuleResolver
Sourcepub fn new() -> FileModuleResolver
pub fn new() -> FileModuleResolver
Create a new FileModuleResolver with the current directory as base path.
The default extension is .rhai.
Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;
// Create a new 'FileModuleResolver' loading scripts from the current directory
// with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new();
let mut engine = Engine::new();
engine.set_module_resolver(resolver);Sourcepub fn new_with_path(path: impl Into<PathBuf>) -> FileModuleResolver
pub fn new_with_path(path: impl Into<PathBuf>) -> FileModuleResolver
Create a new FileModuleResolver with a specific base path.
The default extension is .rhai.
Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;
// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new_with_path("./scripts");
let mut engine = Engine::new();
engine.set_module_resolver(resolver);Sourcepub fn new_with_extension(
extension: impl Into<SmartString<LazyCompact>>,
) -> FileModuleResolver
pub fn new_with_extension( extension: impl Into<SmartString<LazyCompact>>, ) -> FileModuleResolver
Create a new FileModuleResolver with a file extension.
Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;
// Create a new 'FileModuleResolver' loading scripts with file extension '.rhai' (the default).
let resolver = FileModuleResolver::new_with_extension("rhai");
let mut engine = Engine::new();
engine.set_module_resolver(resolver);Sourcepub fn new_with_path_and_extension(
path: impl Into<PathBuf>,
extension: impl Into<SmartString<LazyCompact>>,
) -> FileModuleResolver
pub fn new_with_path_and_extension( path: impl Into<PathBuf>, extension: impl Into<SmartString<LazyCompact>>, ) -> FileModuleResolver
Create a new FileModuleResolver with a specific base path and file extension.
Example
use rhai::Engine;
use rhai::module_resolvers::FileModuleResolver;
// Create a new 'FileModuleResolver' loading scripts from the 'scripts' subdirectory
// with file extension '.x'.
let resolver = FileModuleResolver::new_with_path_and_extension("./scripts", "x");
let mut engine = Engine::new();
engine.set_module_resolver(resolver);Sourcepub fn set_base_path(
&mut self,
path: impl Into<PathBuf>,
) -> &mut FileModuleResolver
pub fn set_base_path( &mut self, path: impl Into<PathBuf>, ) -> &mut FileModuleResolver
Set the base path for script files.
Sourcepub fn set_extension(
&mut self,
extension: impl Into<SmartString<LazyCompact>>,
) -> &mut FileModuleResolver
pub fn set_extension( &mut self, extension: impl Into<SmartString<LazyCompact>>, ) -> &mut FileModuleResolver
Set the script file extension.
Sourcepub fn enable_cache(&mut self, enable: bool) -> &mut FileModuleResolver
pub fn enable_cache(&mut self, enable: bool) -> &mut FileModuleResolver
Enable/disable the cache.
Sourcepub const fn is_cache_enabled(&self) -> bool
pub const fn is_cache_enabled(&self) -> bool
Is the cache enabled?
Sourcepub fn clear_cache(&mut self) -> &mut FileModuleResolver
pub fn clear_cache(&mut self) -> &mut FileModuleResolver
Empty the internal cache.