pub struct ModuleResolversCollection(/* private fields */);Available on crate feature
rhai only.Expand description
Module resolution service that holds a collection of module resolvers, to be searched in sequential order.
Example
use rhai::{Engine, Module};
use rhai::module_resolvers::{StaticModuleResolver, ModuleResolversCollection};
let mut collection = ModuleResolversCollection::new();
let resolver = StaticModuleResolver::new();
collection.push(resolver);
let mut engine = Engine::new();
engine.set_module_resolver(collection);Implementations
Sourceimpl ModuleResolversCollection
impl ModuleResolversCollection
Sourcepub const fn new() -> ModuleResolversCollection
pub const fn new() -> ModuleResolversCollection
Create a new ModuleResolversCollection.
Example
use rhai::{Engine, Module};
use rhai::module_resolvers::{StaticModuleResolver, ModuleResolversCollection};
let mut collection = ModuleResolversCollection::new();
let resolver = StaticModuleResolver::new();
collection.push(resolver);
let mut engine = Engine::new();
engine.set_module_resolver(collection);Sourcepub fn push(
&mut self,
resolver: impl ModuleResolver + 'static,
) -> &mut ModuleResolversCollection
pub fn push( &mut self, resolver: impl ModuleResolver + 'static, ) -> &mut ModuleResolversCollection
Append a module resolver to the end.
Sourcepub fn insert(
&mut self,
index: usize,
resolver: impl ModuleResolver + 'static,
) -> &mut ModuleResolversCollection
pub fn insert( &mut self, index: usize, resolver: impl ModuleResolver + 'static, ) -> &mut ModuleResolversCollection
Sourcepub fn pop(&mut self) -> Option<Box<dyn ModuleResolver>>
pub fn pop(&mut self) -> Option<Box<dyn ModuleResolver>>
Remove the last module resolver from the end, if any.
Sourcepub fn remove(&mut self, index: usize) -> Box<dyn ModuleResolver>
pub fn remove(&mut self, index: usize) -> Box<dyn ModuleResolver>
Sourcepub fn iter(&self) -> impl Iterator<Item = &dyn ModuleResolver>
pub fn iter(&self) -> impl Iterator<Item = &dyn ModuleResolver>
Get an iterator of all the module resolvers.
Sourcepub fn clear(&mut self) -> &mut ModuleResolversCollection
pub fn clear(&mut self) -> &mut ModuleResolversCollection
Remove all module resolvers.
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Returns true if this ModuleResolversCollection contains no module resolvers.
Sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
Get the number of module resolvers in this ModuleResolversCollection.
Sourcepub fn append(
&mut self,
other: ModuleResolversCollection,
) -> &mut ModuleResolversCollection
pub fn append( &mut self, other: ModuleResolversCollection, ) -> &mut ModuleResolversCollection
Add another ModuleResolversCollection to the end of this collection.
The other ModuleResolversCollection is consumed.