CLIENT

Struct CLIENT 

pub struct CLIENT { /* private fields */ }
Available on crate feature k8s only.

Methods from Deref<Target = Client>

pub fn valid_until(&self) -> &Option<Timestamp>

Get the expiration timestamp of the client, if it has been set.

pub fn default_namespace(&self) -> &str

Get the default namespace for the client

The namespace is either configured on context in the kubeconfig, falls back to default when running locally, or uses the service account’s namespace when deployed in-cluster.

pub async fn send( &self, request: Request<Body>, ) -> Result<Response<Body>, Error>

Perform a raw HTTP request against the API and return the raw response back. This method can be used to get raw access to the API which may be used to, for example, create a proxy server or application-level gateway between localhost and the API server.

pub async fn request<T>(&self, request: Request<Vec<u8>>) -> Result<T, Error>

Perform a raw HTTP request against the API and deserialize the response as JSON to some known type.

pub async fn request_text( &self, request: Request<Vec<u8>>, ) -> Result<String, Error>

Perform a raw HTTP request against the API and get back the response as a string

pub async fn request_stream( &self, request: Request<Vec<u8>>, ) -> Result<impl AsyncBufRead + use<>, Error>

Perform a raw HTTP request against the API and stream the response body.

The response can be processed using AsyncReadExt and AsyncBufReadExt.

pub async fn request_status<T>( &self, request: Request<Vec<u8>>, ) -> Result<Either<T, Status>, Error>

Perform a raw HTTP request against the API and get back either an object deserialized as JSON or a Status Object.

pub async fn request_events<T>( &self, request: Request<Vec<u8>>, ) -> Result<impl TryStream + use<T>, Error>

Perform a raw request and get back a stream of WatchEvent objects

pub async fn apiserver_version(&self) -> Result<Info, Error>

Returns apiserver version.

pub async fn list_api_groups(&self) -> Result<APIGroupList, Error>

Lists api groups that apiserver serves.

pub async fn list_api_group_resources( &self, apiversion: &str, ) -> Result<APIResourceList, Error>

Lists resources served in given API group.

Example usage:
let apigroups = client.list_api_groups().await?;
for g in apigroups.groups {
    let ver = g
        .preferred_version
        .as_ref()
        .or_else(|| g.versions.first())
        .expect("preferred or versions exists");
    let apis = client.list_api_group_resources(&ver.group_version).await?;
    dbg!(apis);
}
pub async fn list_core_api_versions(&self) -> Result<APIVersions, Error>

Lists versions of core a.k.a. "" legacy API group.

pub async fn list_core_api_resources( &self, version: &str, ) -> Result<APIResourceList, Error>

Lists resources served in particular core group version.

pub async fn list_api_groups_aggregated( &self, ) -> Result<APIGroupDiscoveryList, Error>

Returns aggregated discovery for all API groups served at /apis.

This uses the Aggregated Discovery API to fetch all non-core API groups and their resources in a single request.

Requires Kubernetes 1.26+ (beta) or 1.30+ (stable).

Example usage:
let discovery = client.list_api_groups_aggregated().await?;
for group in discovery.items {
    let name = group.metadata.as_ref().and_then(|m| m.name.as_ref());
    println!("Group: {:?}", name);
    for version in group.versions {
        println!("  Version: {:?}", version.version);
        for resource in version.resources {
            println!("    Resource: {:?}", resource.resource);
        }
    }
}
pub async fn list_core_api_versions_aggregated( &self, ) -> Result<APIGroupDiscoveryList, Error>

Returns aggregated discovery for core API group served at /api.

This uses the Aggregated Discovery API to fetch the core API group and all its resources in a single request.

Requires Kubernetes 1.26+ (beta) or 1.30+ (stable).

Example usage:
let discovery = client.list_core_api_versions_aggregated().await?;
for group in discovery.items {
    for version in group.versions {
        println!("Core version: {:?}", version.version);
        for resource in version.resources {
            println!("  Resource: {:?} (scope: {:?})", resource.resource, resource.scope);
        }
    }
}